package vocleditor.edit.parts;

import org.eclipse.draw2d.IFigure;
import org.eclipse.draw2d.Label;
import org.eclipse.draw2d.PolylineConnection;
import org.eclipse.draw2d.geometry.Point;
import org.eclipse.emf.common.notify.Adapter;
import org.eclipse.emf.common.notify.Notification;
import org.eclipse.emf.common.notify.Notifier;
import org.eclipse.gef.DragTracker;
import org.eclipse.gef.EditPolicy;
import org.eclipse.gef.Request;
import org.eclipse.gef.RequestConstants;
import org.eclipse.gef.editparts.AbstractGraphicalEditPart;
import org.eclipse.gef.tools.DirectEditManager;
import org.eclipse.jface.viewers.TextCellEditor;
import org.eclipse.ui.views.properties.IPropertySource;

import vocleditor.edit.AssociationEndRoleCellEditorLocator;
import vocleditor.edit.AssociationEndRoleConstraint;
import vocleditor.edit.AssociationEndRoleEditManager;
import vocleditor.edit.AssociationEndRoleTracker;
import vocleditor.edit.policies.AssociationEndRoleDirectEditPolicy;
import vocleditor.edit.policies.AssociationEndRoleMovePolicy;
import vocleditor.model.EObjectPropertySource;
import VOCLModel.AssociationEndRole;
import VOCLModel.AssociationEndRoleSource;
/**
 * Class       : AssociationEndRoleSourceEditPart<br/>
 * Package     : vocleditor.edit.parts<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This editpart represents the role label at the end of an association role<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class AssociationEndRoleSourceEditPart extends AbstractGraphicalEditPart implements Adapter
{
	private IPropertySource propertySource = null;
	private Notifier target;

	private DirectEditManager manager;

	/**
	 * Constructor for AssociationEndRoleEditPart. 
	 */
	public AssociationEndRoleSourceEditPart(AssociationEndRole associationEndRole)
	{
		setModel(associationEndRole);
	}


	
	/**
	 * Creates an Label at the begin of an AssociationRole.
	 * Often it is empty.
	 */
	protected IFigure createFigure()
	{
		AssociationEndRoleSource role = getAssociationEndRoleSource();
		String text = role.getName();
		if (text == null || text.length() == 0)
		{
			text = role.getClassifierNode().getType();
			char c = text.charAt(0);
			if (Character.isUpperCase(c))
			{
				text = Character.toLowerCase(c) + text.substring(1);
			}
		}
		
		Label label = new Label();
		label.setText(text);
		return label;
	}

	/**
	 * called when its necessary to refresh the role label
	 */
	public void notifyChanged(Notification notification)
	{
		refreshVisuals();
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#createEditPolicies()
	 */
	protected void createEditPolicies()
	{
		//EditPolicy for enabling direct editing
		installEditPolicy(EditPolicy.DIRECT_EDIT_ROLE, new AssociationEndRoleDirectEditPolicy());
		installEditPolicy(EditPolicy.GRAPHICAL_NODE_ROLE, new AssociationEndRoleMovePolicy());
	}

	/**
	 * @see org.eclipse.gef.EditPart#getDragTracker(org.eclipse.gef.Request)
	 */
	public DragTracker getDragTracker(Request request)
	{
		return new AssociationEndRoleTracker(this, (AssociationRoleEditPart)getParent());
	}
	
	/**
	 * @see org.eclipse.gef.editparts.AbstractEditPart#refreshVisuals()
	 */
	protected void refreshVisuals()
	{
		AssociationEndRoleSource role = getAssociationEndRoleSource();
		String text = role.getName();

		boolean isEnd = role.getAssociationRole().getAssociationEndRoleTarget().equals(role);

		Point offset = new Point(role.getX(), role.getY());
		Label figure = (Label)getFigure();
		figure.setText(text);
		AssociationRoleEditPart parent = (AssociationRoleEditPart)getParent();
		PolylineConnection connFigure = (PolylineConnection)parent.getFigure();
		AssociationEndRoleConstraint constraint = new AssociationEndRoleConstraint(text, isEnd, offset, connFigure);
		parent.setLayoutConstraint(this, getFigure(), constraint);
	}

	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#activate()
	 */
	public void activate()
	{
		if (isActive())
			return;

		// start listening for changes in the model
		 ((Notifier)getAssociationEndRoleSource()).eAdapters().add(this);
		super.activate();
	}


	/**
	 * @see org.eclipse.gef.editparts.AbstractGraphicalEditPart#deactivate()
	 */
	public void deactivate()
	{
		if (!isActive())
			return;

		// stop listening for changes in the model
		 ((Notifier)getAssociationEndRoleSource()).eAdapters().remove(this);
		super.deactivate();
	}

	private AssociationEndRoleSource getAssociationEndRoleSource()
	{
		return (AssociationEndRoleSource)getModel();
	}

	public Notifier getTarget()
	{
		return target;
	}

	public boolean isAdapterForType(Object type)
	{
		return type.equals(getModel().getClass());
	}

	public void setTarget(Notifier newTarget)
	{
		target = newTarget;
	}

	public Object getAdapter(Class key)
	{
		if (IPropertySource.class == key)
		{
			return getPropertySource();
		}
		return super.getAdapter(key);
	}

	protected IPropertySource getPropertySource()
	{
		if (propertySource == null)
		{
			propertySource = new EObjectPropertySource(getAssociationEndRoleSource());
		}
		return propertySource;
	}

	public void performRequest(Request request)
	{
		if (request.getType() == RequestConstants.REQ_DIRECT_EDIT || request.getType() == RequestConstants.REQ_OPEN)
			performDirectEdit();
	}

	private void performDirectEdit()
	{
		if (manager == null)
			manager =
				new AssociationEndRoleEditManager(
					this,
					TextCellEditor.class,
					new AssociationEndRoleCellEditorLocator((Label)getFigure()));
		manager.show();
	}
}
