
package vocleditor.editor;

import java.util.EventObject;

import org.eclipse.gef.commands.Command;
import org.eclipse.gef.commands.CommandStack;
import org.eclipse.gef.commands.CommandStackListener;
import org.eclipse.gef.commands.UnexecutableCommand;

/**
 * This is a delegating command stack, which delegates everything
 * a defined the CommandStack except event listners.
 * 
 * @author Stefan
 */
public class DelegatingCommandStack
    extends CommandStack
    implements CommandStackListener
{
    private static final Object[] EMPTY_OBJECT_ARRAY = new Object[] {};
    /** the current command stack */
    private CommandStack currentCommandStack;

    /**
     * Returns the current <code>CommandStack</code>.
     * @return the current <code>CommandStack</code>
     */
    public CommandStack getCurrentCommandStack()
    {
        return currentCommandStack;
    }

    /**
     * Sets the current <code>CommandStack</code>.
     * @param stack the <code>CommandStack</code> to set
     */
    public void setCurrentCommandStack(CommandStack stack)
    {
        if (currentCommandStack == stack)
            return;

        // remove from old command stack
        if (null != currentCommandStack)
            currentCommandStack.removeCommandStackListener(this);

        // set new command stack
        currentCommandStack = stack;

        // watch new command stack
        currentCommandStack.addCommandStackListener(this);

        // the command stack changed
        notifyListeners();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#canRedo()
     */
    public boolean canRedo()
    {
        if (null == currentCommandStack)
            return false;

        return currentCommandStack.canRedo();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#canUndo()
     */
    public boolean canUndo()
    {
        if (null == currentCommandStack)
            return false;

        return currentCommandStack.canUndo();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#dispose()
     */
    public void dispose()
    {
        if (null != currentCommandStack)
            currentCommandStack.dispose();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#execute(org.eclipse.gef.commands.Command)
     */
    public void execute(Command command)
    {
        if (null != currentCommandStack)
            currentCommandStack.execute(command);
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#flush()
     */
    public void flush()
    {
        if (null != currentCommandStack)
            currentCommandStack.flush();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#getCommands()
     */
    public Object[] getCommands()
    {
        if (null == currentCommandStack)
            return EMPTY_OBJECT_ARRAY;

        return currentCommandStack.getCommands();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#getRedoCommand()
     */
    public Command getRedoCommand()
    {
        if (null == currentCommandStack)
            return UnexecutableCommand.INSTANCE;

        return currentCommandStack.getRedoCommand();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#getUndoCommand()
     */
    public Command getUndoCommand()
    {
        if (null == currentCommandStack)
            return UnexecutableCommand.INSTANCE;

        return currentCommandStack.getUndoCommand();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#getUndoLimit()
     */
    public int getUndoLimit()
    {
        if (null == currentCommandStack)
            return -1;

        return currentCommandStack.getUndoLimit();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#isDirty()
     */
    public boolean isDirty()
    {
        if (null == currentCommandStack)
            return false;

        return currentCommandStack.isDirty();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#markSaveLocation()
     */
    public void markSaveLocation()
    {
        if (null != currentCommandStack)
            currentCommandStack.markSaveLocation();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#redo()
     */
    public void redo()
    {
        if (null != currentCommandStack)
            currentCommandStack.redo();
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#setUndoLimit(int)
     */
    public void setUndoLimit(int undoLimit)
    {
        if (null != currentCommandStack)
            currentCommandStack.setUndoLimit(undoLimit);
    }

    /**
     * @see org.eclipse.gef.commands.CommandStack#undo()
     */
    public void undo()
    {
        if (null != currentCommandStack)
            currentCommandStack.undo();
    }

    /**
     * @see java.lang.Object#toString()
     */
    public String toString()
    {
        return "DelegatingCommandStack(" + currentCommandStack + ")";
    }

    /**
     * @see org.eclipse.gef.commands.CommandStackListener#commandStackChanged(java.util.EventObject)
     */
    public void commandStackChanged(EventObject event)
    {
        notifyListeners();
    }
}
