
package vocleditor.model;

import java.io.IOException;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;

import org.eclipse.core.runtime.IPath;
import org.eclipse.emf.common.util.EList;
import org.eclipse.emf.common.util.URI;
import org.eclipse.emf.ecore.EPackage;
import org.eclipse.emf.ecore.resource.Resource;
import org.eclipse.emf.ecore.resource.ResourceSet;
import org.eclipse.emf.ecore.resource.impl.ResourceSetImpl;
import org.eclipse.emf.ecore.xmi.XMIResource;
import org.eclipse.emf.ecore.xmi.impl.XMIResourceFactoryImpl;

import VOCLModel.VOCLDiagram;
import VOCLModel.VOCLModelFactory;
import VOCLModel.VOCLModelPackage;
import VOCLModel.impl.VOCLModelPackageImpl;

/**
 * Class       : VOCLModelManager<br/>
 * Package     : vocleditor.model<br/>
 * Project     : VOCLEditor<br/>
 * Created On  : 22.05.2004<br/>
 * Description : This is a manager class for e.g. getting the factory.<br/>
 * <br/>
 * @author Gruppe 1: Stefan Hnsgen, 
 * 					 Andre Hernich,
 * 			 		 Maximilian Kern,
 *		 			 Sebastian Russin<br>
 *
 * 
 */
public class VOCLModelManager {
	/**
	 * For the purpose of the simple editor, a file can only contain a network.
	 * In EMF, a resource provides the way to have access to the model content.
	 */
	private Resource resource = null;

	/**
	 * Contains the factory associated with the model.
	 */
	private static VOCLModelFactory navExpFactory = null;

	/**
	 * Gives access to the top level network contained in the resource.
	 */
	private VOCLDiagram expression = null;

	/**
	 * Returns the resource containing the network. Uses lazy initialization.
	 * @param path
	 * @return
	 */
	public Resource getResource(IPath path) {
		if (resource == null) {
			ResourceSet resSet = getResourceSet();
			resource =
				resSet.getResource(
					URI.createPlatformResourceURI(path.toString()),
					true);
		}
		return resource;
	}

	/**
	 * Creates a resource to contain the network. The resource file does not exist yet.
	 * @param path
	 * @return
	 */
	private Resource createResource(IPath path) {
		if (resource == null) {
			ResourceSet resSet = getResourceSet();
			resource =
				resSet.createResource(
					URI.createPlatformResourceURI(path.toString()));
		}
		return resource;
	}

	/**
	 * Returns the resource set.
	 * @param 
	 * @return
	 */
	private ResourceSet getResourceSet() {
		// Initialize the network package
		VOCLModelPackageImpl.init();
		// Register the XMI resource factory for the .network extension
		Resource.Factory.Registry reg = Resource.Factory.Registry.INSTANCE;
		Map m = reg.getExtensionToFactoryMap();
		m.put("expression", new XMIResourceFactoryImpl());
		// Obtain a new resource set
		return new ResourceSetImpl();
	}

	/**
	 * Returns the factory associated with the model.
	 * Object creation are made through that factory.
	 * @return
	 */
	static public VOCLModelFactory getFactory() {
		if (navExpFactory == null) {
			// Access the factory (needed to create instances)
			Map registry = EPackage.Registry.INSTANCE;
			String networkURI = VOCLModelPackage.eNS_URI;
			VOCLModelPackage networkPackage =
				(VOCLModelPackage) registry.get(networkURI);
			navExpFactory = networkPackage.getVOCLModelFactory();
			
		}
		return navExpFactory;
	}

	/**
	 * Creates a new network.
	 * @param 
	 * @return
	 */
	public VOCLDiagram createVOCLDiagramm(IPath path) {
		createResource(path);
		// Create a new network model
		Map registry = EPackage.Registry.INSTANCE;
		String networkURI = VOCLModelPackage.eNS_URI;
		VOCLModelPackage nPackage = (VOCLModelPackage) registry.get(networkURI);
		VOCLModelFactory nFactory = nPackage.getVOCLModelFactory();
		expression = nFactory.createVOCLDiagram();
		resource.getContents().add(expression);
		return expression;
	}

	/**
	 * Loads the content of the model from the file.
	 * @param path
	 */
	public void load(IPath path) throws IOException {
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
		resource.load(options);
	}

    /**
     * reloads the content of the model from the file.
     * @param path
     */
    public void reload(IPath path) throws IOException {
        getResource(path).unload();
        load(path);
    }

	/**
	 * Saves the content of the model to the file.
	 * @param path
	 */
	public void save(IPath path) throws IOException {
		getResource(path);
		Map options = new HashMap();
		options.put(XMIResource.OPTION_DECLARE_XML, Boolean.TRUE);
		resource.save(options);
	}

	/**
	 * Gets the top level network model.
	 * @return
	 */
	public VOCLDiagram getModel() {
		if (null == expression) {
			EList l = resource.getContents();
			Iterator i = l.iterator();
			while (i.hasNext()) {
				Object o = i.next();
				if (o instanceof VOCLDiagram)
					expression = (VOCLDiagram) o;
			}
		}
		return expression;
	}
}
