/* 
 * $Id: BadElementCollector.java,v 1.2 2004/11/09 09:49:37 haensgen Exp $
 */
 
package VoclToOclConverter;

import java.util.List;
import java.util.Vector;

import org.eclipse.emf.ecore.EObject;

/**
 * Class to collect bad model elements. This class is used by {@link 
 * ModelConverter} and {@link ModelValidator} to store elements of the given 
 * EMF model instance which are invalid in some way. 
 */
public class BadElementCollector {
	
	// The root element of the EMF model instance to store elements for.
	private EObject root = null;

	// The capacity of the collector. This is the maximum number of elements
	// to store.
	private int capacity = 0;
	
	// The vector to hold all elements.
	private Vector elements = null;
	
	/**
	 * Creates a new BadElementCollector with the given capacity.
	 * 
	 * @param capacity
	 *     The capacity the new collector is going to have. This is the 
	 *     maximum number of elements the collector can hold. Values smaller 
	 *     than one will be interpreted as one.
	 */
	public BadElementCollector(EObject root, int capacity) {
		this.root = root;
		if (capacity < 1)
			capacity = 1;
		this.capacity = capacity;
		elements = new Vector(capacity);
	}
	
	/**
	 * Returns the root element of the EMF model instance to store elements for.
	 * 
	 * @return
	 *     The root element.
	 */
	public EObject getRoot() {
		return root;
	}
	
	/**
	 * Returns the capacity of the collector. This is the maximum number of 
	 * elements the collector can hold.
	 * 
	 * @return
	 *     The capacity of the collector.
	 */
	public int getCapacity() {
		return capacity;
	}
	
	/**
	 * Returns the list of elements stored in the collector.
	 * 
	 * @return 
	 *     A list of ElementStatus instances which represent the states of 
	 *     all stored elements.
	 */
	public List getElements() {
		return elements;
	}
	
	/**
	 * Adds a new element to the collector. If the capacity is reached, this 
	 * method does nothing.
	 * 
	 * @param code
	 *     The error code. See {@link IElementStatusConstants} for such codes.
	 * @param message
	 *     An message to describe the status of the element.
	 * @param element
	 *     The element to add.
	 */
	public void addElement(int code, String message, EObject element) {
		if (elements.size() < capacity)
			elements.add(new ElementStatus(code, message, root, element));
	}
	
	/**
	 * Removes all elements from the collector.
	 */
	public void reset() {
		elements.removeAllElements();
	}
}
